import nestargs
import pytest


class TestNestedArgumentParser:
    def test_parse_args(self):
        parser = nestargs.NestedArgumentParser()
        parser.add_argument("some.a")
        parser.add_argument("some.b")
        parser.add_argument("some.c.d")

        args = parser.parse_args(["1", "2", "3"])
        assert args.some.a == "1"
        assert args.some.b == "2"
        assert args.some.c.d == "3"
        assert vars(args.some).keys() == {"a", "b", "c"}
        assert vars(args.some.c).keys() == {"d"}

    def test_parse_args_with_another_delimiter(self):
        parser = nestargs.NestedArgumentParser(delimiter="/")
        parser.add_argument("some/a")
        parser.add_argument("some/b")
        parser.add_argument("some/c/d")

        args = parser.parse_args(["1", "2", "3"])
        assert args.some.a == "1"
        assert args.some.b == "2"
        assert args.some.c.d == "3"
        assert vars(args.some).keys() == {"a", "b", "c"}
        assert vars(args.some.c).keys() == {"d"}

    def test_parse_args_with_empty_parent(self):
        parser = nestargs.NestedArgumentParser()
        parser.add_argument(".empty")

        with pytest.raises(ValueError):
            parser.parse_args([""])



import argparse

DEFAULT_DELIMITER = "."  # default delimiter for nested namespaces


def create_namespace(delimiter: str):
    class _NestedNamespace(argparse.Namespace):
        def __setattr__(self, name, value):
            if delimiter in name:
                parent, name = name.split(delimiter, maxsplit=1)
            else:
                parent = None

            if parent:
                if not hasattr(self, parent):
                    super().__setattr__(parent, self.__class__())
                setattr(getattr(self, parent), name, value)
            elif parent is not None:
                raise ValueError("parent should not be empty: {}".format(name))
            else:
                super().__setattr__(name, value)

    return _NestedNamespace()


class NestedArgumentParser(argparse.ArgumentParser):
    """ArgumentParser that supports nested namespaces."""

    def __init__(self, delimiter: str = DEFAULT_DELIMITER, **kwargs):
        """Initialize NestedArgumentParser.

        Args:
            delimiter: Delimiter for nested namespaces.
            **kwargs: Keyword arguments for argparse.ArgumentParser.
        """
        self.delimiter = delimiter
        super().__init__(**kwargs)

    def parse_known_args(self, args=None, namespace=None):
        if namespace is None:
            namespace = create_namespace(self.delimiter)
        return super().parse_known_args(args=args, namespace=namespace)
